#include <iostream>
#include <cmath>
#include <cstdlib> //For rand
using namespace std;

/*
We want to invert the function y = x*exp(x). That is, we want to input y
and find the corresponding x. This program calculates y for x between 0
and 100 in steps of 1e-5 and saves the values in an array (usually
referred to as a "lookup table" (LUT) in these situations). When the user
inputs a y, the program searches the LUT for the closest value greater
than y and returns the corresponding x. The code compiles and looks like
it should work, but when you run the program it crashes.

1) Fix the program so that it runs correctly without crashing, of course
keeping the intended precision of the lookup table.

2) Improve the program! The original programmer wasn't very clever. I'm
sure you can find many things that could be done better.
*/

const int nPoints = 10000000; //Number of points in the LUT. This sets the LUT precision.
const double xMax = 100;
const int nSearches = 500;

double searchInTable(double y, double* table) { //Search for the first point in the table that is greater than y
	for(int i = 0; i < nPoints; ++i) if(table[i] > y) return i*xMax/nPoints;
}

int main() {
	double yVal[nPoints]; //Array to store the y-values
	double x = 0.;
	for(int i = 0; i < nPoints; ++i) {
		yVal[i] = x*exp(x); //For each x, calculate y and store it here
		x += xMax/nPoints;
	}
	
	//Now let's search in the LUT many times
	double yMax = xMax*exp(xMax);
	cout << "Inverting y = x*exp(x) " << nSearches << " times..." << endl;
	for(int i = 0; i < nSearches; ++i) {
		searchInTable(rand()*yMax/RAND_MAX, yVal); //RAND_MAX is the maximum integer returned by rand()
		if(i % (nSearches/10) == 0) cout << i << endl; //This is just a progress indicator
	}
	cout << "Done!" << endl;
	
	return 0;
}

