#include "graph.h"
using namespace std;

/*
This function should just initialize an empty map. You may point out
that initializing an empty map is exactly what a compiler-provided
default constructor would have done. That's true, but the compiler only
provides a default constructor if we define no constructors at all. In
this case, we have defined a copy constructor as well as two "usual"
constructors. This causes the default constructor to not be defined
unless we define it ourselves. Not having a default constructor is an
error, so writing this function ourselves is really necessary.

In general, it's a good idea to always write a default constructor even
in cases where the compiler would have provided one. This shows the user
that you did not simply forget about it.
*/
graph::graph() {
	
}

/*
Just copy data into the map owned by this graph. You can use an 
initialization list for this!
*/
graph::graph(map<double, double> data) {
	
}

/*
Put the points defined by the x- and y-axes into the map. Remember to
check that xAxis and yAxis are the same size! If they are not, the user
has probably made a mistake so you should refuse to fill the map and
print an error message instead.
*/
graph::graph(vector<double> xAxis, vector<double> yAxis) {
	
}

/*
Put every point in toCopy into the map. You might find it convenient to
implement the add function first.
*/
graph::graph(graph& toCopy) {
	
}

/*
There are no variables to delete. There are no cleanup tasks to do.
Until we implement some more fancy functionalities, we can just leave 
the destructor empty.
*/
graph::~graph() {
	
}

/*
This function should insert all the points in toAdd into the map. You
could, for example, get the map from toAdd using getMap, iterate through
it and either insert or add (using the add function) each point to this
graph's map.
*/
void graph::add(graph& toAdd) {
	
}

/*
This function should simply insert the desired point into the map.
*/
void graph::add(double x, double y) {
	
}

/*
The graph should be able to return the y-value at any point along the
x-axis. For x-values that are between two points, you will have to
interpolate. Therefore, this function should find the two closest points
before and after x, and then use the interpolate function to get the 
value at x. Start by implementing interpolate().

To search in a map, you can for example use upper_bound(x) which returns
an iterator pointing to the first element greater than x, or to end if x
is not in the map. If you instead want to search for an element greater
than or equal to x, use lower_bound(x).
*/
double graph::evaluate(double x) {
	return 0.;
}

/*
The mean value of the graph is just the sum of all the y-values divided
by the number of points in the graph (aka the size of the graph).
*/
double graph::mean() {
	return 0.;
}

/*
You already know how to calculate the standard deviation for a sample
of data points (check the "Significant points" exercise).
*/
double graph::stdev() {
	return 0.;
}

/*
If the points in the graph are equidistant, approximating the integral 
is almost trivial. Imagine a box around each point. The height of the 
box is the y-value of the point and the width of the box is the distance
between two points. Sum up the areas of all boxes to get the integral.
This is called the Trapezoidal rule.

If the points are not equidistant, you have to get smarter. One method
would be to let each box start at the location of the point and extend
all the way to the next point. Of course, this would be a very bad 
approximation under some circumstances.

A more accurate method (and probably easier to code, too) would be to
interpolate between the points. Using interpolation, you could calculate
the y-value for any point along the x-axis. Then it is a simple matter
to get back to a situation with equidistant points! Just divide the
range covered by the graph into, say, 1000 equidistant points. Then use
interpolation to get the y-value for each of those points. Now you can
use the equidistant method again. I suggest implementing the range and
interpolate functions before writing this.

Actually, always using 1000 points is not a good idea (hard-coding
numbers never is). If the real graph has a million points, using only
1000 for the interpolation would not yield an accurate result. Better
then to use the size of the graph as a basis for calculating the number
of points needed for the interpolation. I suggest using 100*size()
points.
*/
double graph::integral() {
	return 0.;
}

/*
start and stop are references. The caller will put two empty variables
here, and you should fill them with the correct values.
*/
void graph::range(double& start, double& stop) {
	start = 0.;
	stop = 0.;
}

/*
To interpolate between two points (x1, y1) and (x2, y2), simply draw
a straight line between them. Once you have the equation for this line,
you can evaluate x at any point along the line. What's that you say?
You forgot the equation for a straight line? Here, I'll remind you:

y = k*x + m

k = (y2 - y1)/(x2 - x1)
m = y - k*x = y1 - k*x1

Of course, you don't really need to calculate m, do you? Just start at
y1 and take a step of length (x - x1) with slope k.
*/
double graph::interpolate(double x, double x1, double y1, double x2, double y2) {
	return 0.;
}
