#ifndef GRAPH_H
#define GRAPH_H

#include <map>
#include <vector>
#include <utility>

class graph {
	public:
	graph(); //Default constructor
	graph(std::map<double, double> data); //Construct from map
	graph(std::vector<double> xAxis, std::vector<double> yAxis); //Construct from two vectors
	graph(graph& toCopy); //Copy constructor
	~graph(); //Destructor
	
	void add(graph& toAdd); //Add the points in another graph to this graph
	void add(double x, double y); //Add a point to this graph
	double evaluate(double x); //Return the y-value corresponding to this x
	double mean(); //Return the mean of the graph
	double stdev(); //Return the standard deviation of the graph
	double integral(); //Return the integral of the graph
	void range(double& start, double& stop); //Return the start and stop of the graph
	std::size_t size() { return data_.size(); } //Return the size of the graph
	std::map<double, double>& getMap() { return data_; } //Return a reference to the internal data
	
	private:
	double interpolate(double x, double x1, double y1, double x2, double y2); //Interpolate between two points
	
	std::map<double, double> data_;
};

#endif
