#include "maxVal.h"
using namespace std;

/*
Finding the maximum of an unordered container can't be done without
looking at every value. Pick the first value as the current maximum,
then loop through the container and compare each value to the current
maximum. If the value is greater than the current max, update it.
The reason we initialize the current max to the first value in the
container rather than e.g 0 or -99999 is that we have no a priori
knowledge of what values are in the contaier (what if they are -1e99)?
*/
int maxVal(vector<int>& toSearch) {
	if(!toSearch.size()) return 0; //If there's nothing to search, return 0
	int currentMax = toSearch.at(0);
	for(size_t i = 1; i < toSearch.size(); ++i) {
		if(toSearch.at(i) > currentMax) currentMax = toSearch.at(i);
	}
	return currentMax;
}


int maxVal(list<int>& toSearch) {
	if(!toSearch.size()) return 0; //If there's nothing to search, return 0
	int currentMax = *toSearch.begin(); //Use an iterator to traverse the list
	for(list<int>::iterator it = toSearch.begin(); it != toSearch.end(); ++it) {
		if(*it > currentMax) currentMax = *it;
	}
	return currentMax;
}

/*
Sets and maps are ordered. We simply have to return the last element.
There are two ways to get it, keeping in mind that we have to access
the elements using iterators. The iterator returned by begin() points to
the first element and the iterator returned by end() points one step
PAST the last element. To get the last element, the fastest way is to
take the iterator returned by end() and decrement it - Then it points to
the last element in the container!

Alternatively, we could use reverse iterators. They work just like
regular iterators except they traverse the container backwards.
Therefore, rbegin() points to the last element and rend() points one
step "past" the first element. To get the last element, one could
simply take what is pointed to by rbegin(). This uses fewer lines of
code and we don't have to waste an operation decrementing an iterator.
Both approaches are shown below, with set using the reverse iterator.
*/
int maxVal(set<int>& toSearch) {
	if(!toSearch.size()) return 0; //If there's nothing to search, return 0
	return *toSearch.rbegin(); //Return what it pointed to by the reverse begin iterator
}

int maxVal(map<int, int>& toSearch) {
	if(!toSearch.size()) return 0;
	map<int, int>::iterator it = toSearch.end(); //Points one step past the last element
	--it; //Decrement it - Now it points to the last element!
	return it->second; //Return the value in the key-value pair
}
