#include "graph.h"
#include <iostream>
#include <vector>
#include <map>
#include <cmath>

using namespace std;

/*
We'll create three graphs using different creation methods, and then
test that the graphs are working. One graph will be empty, the others
will contain the functions y = sin(x) and y = 2x.
*/
int main() {
	map<double, double> emptyMap;
	graph graphMap(emptyMap); //An empty graph created from an empty map
	
	graph graphCopy(graphMap); //Create using the existing empty graph
	graphCopy.add(0., 1.); //Add a single point
	
	double pi = 3.1415926535;
	int nPoints = 1000;
	vector<double> xAxis;
	vector<double> yAxis;
	for(int i = 0; i < nPoints; ++i) {
		xAxis.push_back(2*pi*i/nPoints); //Linear x-axis from 0 to 2*pi in nPoints steps
		yAxis.push_back(sin(xAxis.back())); //xAxis.back() returns the value at the back of the vector
	}
	graph graphVec(xAxis, yAxis); //Graph of sin(x) created from two vectors
	
	/*
	You can create a logarithmic axis going from some min to some max in
	N + 1 steps by finding the multiplication factor x that fulfills
	
	max = min*x^N
	
	Solving for x yields x = exp(log(max/min)/N). Try it yourself!
	Why do I say that it's N + 1 steps and not N? Because I want to
	include the "zeroth" step in my range. That is, I want the range to
	start at min*x^0 = min and then reach max with N factors of x. That
	would be a total of N + 1 points.
	*/
	
	double x = 1./nPoints;
	double multFactor = exp(log(10./x)/(nPoints - 1)); //min to max with 999 factors of x
	graph graphDef; //Create an empty graph using the default constructor, then fill it with the add function
	for(int i = 0; i < nPoints; ++i) {
		graphDef.add(x, 2.*x); //Graph of y = 2*x with a logarithmic x-axis
		x *= multFactor;
	}
	
	cout << "Testing empty graph" << endl;
	cout << "size " << graphMap.size() << endl;
	cout << "mean " << graphMap.mean() << endl;
	cout << "stdev " << graphMap.stdev() << endl;
	cout << "integral " << graphMap.integral() << endl;
	cout << "f(-0.1) " << graphMap.evaluate(-0.1) << endl;
	cout << "f(pi/2) " << graphMap.evaluate(pi/2) << endl;
	cout << "f(15) " << graphMap.evaluate(15) << endl << endl;
	
	cout << "Testing single point (0, 1)" << endl;
	cout << "size " << graphCopy.size() << endl;
	cout << "mean " << graphCopy.mean() << endl;
	cout << "stdev " << graphCopy.stdev() << endl;
	cout << "integral " << graphCopy.integral() << endl;
	cout << "f(-0.1) " << graphCopy.evaluate(-0.1) << endl;
	cout << "f(pi/2) " << graphCopy.evaluate(pi/2) << endl;
	cout << "f(15) " << graphCopy.evaluate(15) << endl << endl;
	
	cout << "Testing f(x) = sin(x)" << endl;
	cout << "size " << graphVec.size() << endl;
	cout << "mean " << graphVec.mean() << endl;
	cout << "stdev " << graphVec.stdev() << endl;
	cout << "integral " << graphVec.integral() << endl;
	cout << "f(-0.1) " << graphVec.evaluate(-0.1) << endl;
	cout << "f(pi/2) " << graphVec.evaluate(pi/2) << endl;
	cout << "f(15) " << graphVec.evaluate(15) << endl << endl;
	
	cout << "Testing f(x) = 2x" << endl;
	cout << "size " << graphDef.size() << endl;
	cout << "mean " << graphDef.mean() << endl;
	cout << "stdev " << graphDef.stdev() << endl;
	cout << "integral " << graphDef.integral() << endl;
	cout << "f(-0.1) " << graphDef.evaluate(-0.1) << endl;
	cout << "f(pi/2) " << graphDef.evaluate(pi/2) << endl;
	cout << "f(15) " << graphDef.evaluate(15) << endl << endl;
	return 0;
}
