#include <iostream>
#include <fstream>
#include <set>
#include <algorithm> //For set_difference

using namespace std;

int main(int argc, char *argv[]) {
	ifstream inFile1(argv[1]); //Open two files
	if(!inFile1) {
		cout << "Can't open dataSetFile " << argv[1] << ". Exiting." << endl;
		return 1;
	}
	ifstream inFile2(argv[2]);
	if(!inFile2) {
		cout << "Can't open dataSetFile " << argv[2] << ". Exiting." << endl;
		return 1;
	}
	set<int> events1; //Create two set to hold the ints
	set<int> events2;
	int temp = 0;

	while(inFile1 >> temp) events1.insert(temp); //Read in the ints. Now we want to find the set differences!
	while(inFile2 >> temp) events2.insert(temp);
	inFile1.close();
	inFile2.close();
	
	set<int> in1ButNotIn2; //Method 1, iterate through the sets and compare
	set<int> in2ButNotIn1; //This method is not so fast because we search through the entire set for each comparison
	for(set<int>::iterator it = events1.begin(); it != events1.end(); ++it) if(events2.find(*it) == events2.end()) in1ButNotIn2.insert(*it);
	for(set<int>::iterator it = events2.begin(); it != events2.end(); ++it) if(events1.find(*it) == events1.end()) in2ButNotIn1.insert(*it);

	//set<int> in1ButNotIn2; //Method 2, use STL magic
	//set<int> in2ButNotIn1; //The STL algo is faster because it requires the inputs to be ordered and then exploits that fact
	//set_difference(events1.begin(), events1.end(), events2.begin(), events2.end(), std::inserter(in1ButNotIn2, in1ButNotIn2.end()));
	//set_difference(events2.begin(), events2.end(), events1.begin(), events1.end(), std::inserter(in2ButNotIn1, in2ButNotIn1.end()));

	cout << "In file 1 but not in file 2:" << endl;
	for(set<int>::iterator it = in1ButNotIn2.begin(); it != in1ButNotIn2.end(); ++it) cout << *it << endl;
	cout << "In file 2 but not in file 1:" << endl;
	for(set<int>::iterator it = in2ButNotIn1.begin(); it != in2ButNotIn1.end(); ++it) cout << *it << endl;
}
